@@ -55,15 +55,16 @@ AUTH0_CLIENT_ID = environ.get("AUTH0_CLIENT_ID")
 ALGORITHMS = ["RS256"]
 
 ### Configure GCP ###
-GOOGLE_CLOUD_PROJECT = environ.get("GOOGLE_CLOUD_PROJECT")
-GOOGLE_UPLOAD_BUCKET = environ.get("GOOGLE_UPLOAD_BUCKET")
-GOOGLE_UPLOAD_TOPIC = environ.get("GOOGLE_UPLOAD_TOPIC")
-GOOGLE_DATA_BUCKET = environ.get("GOOGLE_DATA_BUCKET")
-GOOGLE_UPLOAD_ROLE = environ.get("GOOGLE_UPLOAD_ROLE")
+GOOGLE_CLOUD_PROJECT = environ["GOOGLE_CLOUD_PROJECT"]
+GOOGLE_INTAKE_BUCKET = environ["GOOGLE_INTAKE_BUCKET"]
+GOOGLE_UPLOAD_BUCKET = environ["GOOGLE_UPLOAD_BUCKET"]
+GOOGLE_UPLOAD_TOPIC = environ["GOOGLE_UPLOAD_TOPIC"]
+GOOGLE_DATA_BUCKET = environ["GOOGLE_DATA_BUCKET"]
+GOOGLE_UPLOAD_ROLE = environ["GOOGLE_UPLOAD_ROLE"]
 GOOGLE_DOWNLOAD_ROLE = "roles/storage.objectViewer"  # same across environments
-GOOGLE_PATIENT_SAMPLE_TOPIC = environ.get("GOOGLE_PATIENT_SAMPLE_TOPIC")
-GOOGLE_EMAILS_TOPIC = environ.get("GOOGLE_EMAILS_TOPIC")
-GOOGLE_ARTIFACT_UPLOAD_TOPIC = environ.get("GOOGLE_ARTIFACT_UPLOAD_TOPIC")
+GOOGLE_PATIENT_SAMPLE_TOPIC = environ["GOOGLE_PATIENT_SAMPLE_TOPIC"]
+GOOGLE_EMAILS_TOPIC = environ["GOOGLE_EMAILS_TOPIC"]
+GOOGLE_ARTIFACT_UPLOAD_TOPIC = environ["GOOGLE_ARTIFACT_UPLOAD_TOPIC"]
 # This is a limit set by GCP - there will never be more than this many
 # conditional bindings for a single member-role combo.
 # See: https://cloud.google.com/iam/docs/conditions-overview
